{% extends 'freight/base.html' %}
{% load i18n %}
{% load humanize %}
{% load bootstrap %}

{% block details %}
    <form class="form" action="{% url 'freight:add_location_2' %}" method="POST">
        {% csrf_token %}
        {{ form|bootstrap }}        
        <button class="btn btn-success btn-block" type="submit">Add / Update Location</button>
        <p class="text-muted">Accessing structures with token from <strong>{{ token_char_name }}</strong></p>
    </form>    
{% endblock %}