# Generated by Django 2.2.5 on 2019-10-30 15:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("freight", "0007_auto_20191029_1803"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="contracthandler",
            options={
                "verbose_name": "Contract Handler [my_corporation]",
                "verbose_name_plural": "Contract Handler [my_corporation]",
            },
        ),
        migrations.AlterField(
            model_name="contracthandler",
            name="last_error",
            field=models.IntegerField(
                choices=[
                    (0, "No error"),
                    (1, "Invalid token"),
                    (2, "Expired token"),
                    (3, "Insufficient permissions"),
                    (4, "No character set for fetching alliance contacts"),
                    (5, "ESI API is currently unavailable"),
                    (6, "Operaton mode does not match with current setting"),
                    (99, "Unknown error"),
                ],
                default=0,
                help_text="error that occurred at the last sync atttempt (if any)",
            ),
        ),
    ]
